unit GLModelManager;

(* GLSM added: ILH, 12/06/2004
 * Path Deleted, Matlib added: ILH, 11/12/2004
 * Author : Blaise Bernier
 * Date   : 08/12/2003
 *
 ******************************************************************************
 * Description :
 *    The GLModelManager is a class that provides an easy way to manage
 *    models and master objects of proxies at runtime.  You just need to
 *    have the model filename and it will load it into the list or return
 *    the already existing model.
 *
 *
 *)

interface

uses GLScene, GLObjects, GLVectorFileObjects, SysUtils, Classes,
GLTexture,
Dialogs,//Showmessage
     GLFile3DS, GLFileOBJ;

type

   (****************************************
      TGLModelManager - class definition
   ****************************************)
   TGLModelManager = class
   private
      { Private fields }
      FMasterObject : TGLDummyCube;
      FModelList    : TStringList;
      FMasterMatLib : TGLMaterialLibrary;
   public
      { Constructor / Destructor }
      constructor Create(AMasterDC : TGLDummyCube; AMasterMatLib : TGLMaterialLibrary); virtual;
      destructor Destroy; override;

      { Methods }
      function LoadModel(AFilename,TexFilename : string): TGLFreeForm;

      { Properties }
      property MasterObject : TGLDummyCube read FMasterObject;
      property ModelList : TStringList read FModelList;
      property MasterMatLib : TGLMaterialLibrary read FMasterMatLib;
   end;


implementation

(****************************************
         { TGLModelManager }
****************************************)

(**********************************
   Name        : Create
   Description :
      This is the constructor.  It will create the list, assign the path to the
      models and the master object where the new models will be loaded.
*)
constructor TGLModelManager.Create(AMasterDC: TGLDummyCube; AMasterMatLib : TGLMaterialLibrary);
begin
   // Set the master object
   FMasterObject := AMasterDC;
   FMasterMatLib:=AMasterMatLib;
   // Create the model list
   FModelList := TStringList.Create;
  // FModelList.CaseSensitive := false;
   FModelList.Sorted := true;
end;

(**********************************
   Name        : Destroy
   Description :
      This is the destructor.  It will free all the loaded models.
*)
destructor TGLModelManager.Destroy;
var
   i : integer;
begin
   // Destroy every models
   for i:=0 to Pred(FModelList.Count) do
      FModelList.Objects[i].Destroy;
   // Destroy the list
   FModelList.Destroy;
   inherited;
end;

(**********************************
   Name        : LoadModel
   Description :
      It will load a new model if it's not in the list and then return the new
      freeform.  If it,s already in the list, it will return the existing
      freeform.
*)
function TGLModelManager.LoadModel(AFilename,TexFilename : string): TGLFreeForm;
var
   I           : integer;
   NewFreeForm : TGLFreeForm;
   libMat : TGLLibMaterial;
begin
   with FModelList do
   begin
      if Find(ExtractFileName(AFilename), I) then
         Result := TGLFreeForm(Objects[I])
      else
      begin
   if Uppercase(ExtractFileExt(AFilename))='.GLSM' then
   begin
        try
           if FileExists(ChangeFileExt(AFilename,'.GLML')) then
              FMasterMatLib.LoadFromFile(ChangeFileExt(AFilename,'.GLML'));
        except
              ShowMessage('Error loading MaterialLibrary "'+ChangeFileExt(AFilename,'.GLML')+'"');
        end;

   end;

         NewFreeForm := TGLFreeForm(FMasterObject.AddNewChild(TGLFreeForm));
         NewFreeForm.MaterialLibrary:= FMasterMatLib;
         NewFreeForm.LoadFromFile(AFilename);
         If FileExists(TexFilename) then
         begin
           libMat:=FMasterMatLib.Materials.Add;
           libMat.Name:=ExtractFileName(TexFilename);
           NewFreeForm.Material.LibMaterialName:=libMat.Name;
           NewFreeForm.Material.MaterialLibrary:=FMasterMatLib;
           FMasterMatLib.materials.GetLibMaterialByName(ExtractFileName(TexFilename)).material.texture.image.loadfromfile(TexFilename);
         end;
         FModelList.AddObject(ExtractFileName(AFilename),NewFreeForm);
         Result := NewFreeForm;
      end;
   end;

end;



end.
