unit TreeMatrix;
(*
 | Deze unit bevat een definitie van een matrix class. Standaard operaties op
 | matrices zijn gemplementeerd, zoals het vermenigvuldigen en optellen van
 | matrices. Deze unit is niet compleet, maar bevat die functionaliteit nodig
 | voor WhirlWind, een raytracer. Zo zijn de elementen van de matrix beperkt
 | tot het type Real. Bovendien heeft de matrix standaard de afmetingen 4x4,
 | aangezien dit is wat we nodig hebben wanneer we matrices gaan gebruiken
 | bij 3D geometrie in het homogene cordinatenstelsel.
 |                                                       (c) Ton van den Heuvel
 |                                                                   17-04-2001
 *)

{$IFDEF DEBUG}
  {$ASSERTIONS ON}
{$ENDIF}

interface

uses Classes, TreeVector;

type TAxis = (aX, aY, aZ);

     TMatrix = class(TObject)
     private
       FElements: array[0..3] of array[0..3] of real;
     public
       constructor Create; overload;

       function Element(const i, j: integer) : real;
       function Add(const AMatrix: TMatrix): TMatrix; overload;
       function Add(const C: Real): TMatrix; overload;
       function Subtract(const AMatrix: TMatrix): TMatrix; overload;
       function Subtract(const C: real): TMatrix; overload;
       function Multiply(const AMatrix: TMatrix): TMatrix; overload;
       function Multiply(const C: real): TMatrix; overload;
       function Multiply(const APoint3d: TPoint3d): TPoint3d; overload;
       function Multiply(const AVector: TVector3d): TVector3d; overload;

       procedure SetIdentityMatrix;
       procedure ZeroMatrix;
       procedure SetElement(const i,j: Integer; const v: real);
       procedure SetTranslateMatrix(const tx, ty, tz: real);
       procedure SetScaleMatrix(const sx, sy, sz: real);
       procedure SetRotationMatrix(const Angle: real; const AAxis: TAxis);
       procedure SetColumn(const i: integer; const AVector: TVector3d); overload;
       procedure SetColumn(const i: integer; const APoint: TPoint3d); overload;
       procedure SetColumn(const i: integer; const x, y, z: real); overload;
       procedure SetRow(const i: integer; const AVector: TVector3d); overload;
       procedure SetRow(const i: integer; const APoint: TPoint3d); overload;
       procedure SetRow(const i: integer; const x, y, z: real); overload;
       procedure Assign(const ASourceMatrix: TMatrix); virtual;
     end;

     {
        Representatieinvariant: Matrix.m: #m.FElements = m.FRows*m.FColumns
        Abstractiefunctie: [[m(i, j)]] = m.FElements[i,j]
     }

implementation

constructor TMatrix.Create;
begin
  inherited Create;
  ZeroMatrix;
end;

procedure TMatrix.SetElement(const i, j: Integer; const v: Real);
begin
  FElements[i, j] := v;
end;

function TMatrix.Add(const AMatrix: TMatrix): TMatrix;
var i, j: Integer;
begin
  Result := TMatrix.Create;

  { Doorloop alle elementen (i, j) van de Result matrix en ken de som van de
    elementen (i, j) van de matrix Self en AMatrix aan het element (i, j) van
    de Result matrix toe. }

  for i := 0 to 3 do
    for j := 0 to 3 do
      Result.SetElement(i, j, Self.Element(i, j) + AMatrix.Element(i, j));
end;

function TMatrix.Add(const C: Real): TMatrix;
var i, j: Integer;
begin
  Result := TMatrix.Create;

  { Doorloop alle elementen (i, j) van de Result matrix en ken de som van het
    element (i, j) van de Self matrix en C aan het element (i, j) van de Result
    matrix toe. }

  for i := 0 to 3 do
    for j := 0 to 3 do
      Result.SetElement(i, j, Self.Element(i, j) + C);
end;

function TMatrix.Subtract(const AMatrix: TMatrix): TMatrix;
var i, j: Integer;
begin
  Result := TMatrix.Create;

  { Doorloop alle elementen (i, j) van de Result matrix en ken het verschil van
    de elementen (i, j) van de matrix Self en AMatrix aan het element (i, j)
    van de Result matrix toe. }

  for i := 0 to 3 do
    for j := 0 to 3 do
      Result.SetElement(i, j, Self.Element(i, j) - AMatrix.Element(i, j));
end;

function TMatrix.Subtract(const C: Real): TMatrix;
var i, j: Integer;
begin
  Result := TMatrix.Create;

  { Doorloop alle elementen (i, j) van de Result matrix en ken het verschil
    van het element (i, j) van de Self matrix en C aan het element (i, j) van
    de Result matrix toe. }

  for i := 0 to 3 do
    for j := 0 to 3 do
      Result.SetElement(i, j, Self.Element(i, j) - C);
end;

function TMatrix.Multiply(const AMatrix: TMatrix): TMatrix;
var i, j, k: Integer;
begin
  Result := TMatrix.Create;

  { Het onderstaande algoritme is van de orde O(n^3). Eerst worden alle elemen-
    ten van de linkermatrix doorlopen, vervolgens achtereenvolgens alle kolom-
    men van de rechtermatrix. Vervolgens wordt het element van de i-de
    rij en de k-de kolom van de linkermatrix met het element uit de k-de rij
    en de j-de kolom van de rechtermatrix vermenigvuldigt. }

  for i := 0 to 3 do
  { Doorloop alle rijen van de linkermatrix }
    for j := 0 to 3 do
    { Doorloop alle kolommen van de rechtermatrix }
      for k := 0 to 3 do
      { Doorloop alle kolommen van de linkermatrix }
        Result.SetElement(i, j, Result.Element(i, j) + Self.Element(i, k) * AMatrix.Element(k, j));
end;

function TMatrix.Multiply(const C: Real): TMatrix;
var i, j: Integer;
begin
  Result := TMatrix.Create;

  { Doorloop ieder element uit de linkermatrix, i.e. Self, en vermenigvulig
    dit element met C. Sla vervolgens het product op in de Result matrix. }

  for i := 0 to 3 do
    for j := 0 to 3 do
      Result.SetElement(i, j, Self.Element(i, j) * C);
end;

function TMatrix.Multiply(const APoint3d: TPoint3d): TPoint3d;
var i: Integer;
begin
  Result := TPoint3d.Create;

  { Voer een standaard matrix/vector vermenigvuldiging aan. Voor bv. matrix M
    en vector V geldt dan dat Result = M * V. }

  for i := 0 to 3 do
    Result[i] := Self.Element(i, 0) * APoint3d[0] +
                 Self.Element(i, 1) * APoint3d[1] +
                 Self.Element(i, 2) * APoint3d[2] +
                 Self.Element(i, 3) * APoint3d[3];
end;

function TMatrix.Multiply(const AVector: TVector3d): TVector3d;
var i, j: Integer;
begin
  Result := TVector3d.Create;

  { Voer een standaard matrix/vector vermenigvuldiging aan. Voor bv. matrix M
    en vector V geldt dan dat Result = M * V. }

  for i := 0 to 3 do
    for j := 0 to 3 do
      Result[i] := Result[i] + Self.Element(i, j) * AVector[j];
end;

procedure TMatrix.ZeroMatrix;
{ Ken aan alle elementen van de matrix Self de waarde nul toe. }
var i, j: Integer;
begin
  for i := 0 to 3 do
    for j := 0 to 3 do
      SetElement(i, j, 0);
end;

procedure TMatrix.SetIdentityMatrix;
{ Maak van de huidige matrix een identiteitsmatrix, i.e. een matrix met alle
  elementen gelijk aan nul, behalve de elementen op de diagonaal, die gelijk
  zijn aan 1. }
var i: Integer;
begin
  ZeroMatrix;
  for i := 0 to 3 do SetElement(i, i, 1);
end;

procedure TMatrix.SetTranslateMatrix(const tx, ty, tz: Real);
{ Maak van de huidige matrix een translatie transformatie matrix. tx, ty en
  tz geven de waarden aan voor de translatie in respectievelijk x, y en z
  richting. }
begin
  SetIdentityMatrix;
  Self.SetElement(0, 3, tx);
  Self.SetElement(1, 3, ty);
  Self.SetElement(2, 3, tz);
end;

procedure TMatrix.SetScaleMatrix(const sx, sy, sz: Real);
{ Maak van de huidige matrix een scale transformatie matrix. sx, sy en sz
  geven de waarden aan voor de schaling in respectievelijk x, y en z
  richting. }
begin
  ZeroMatrix;
  Self.SetElement(0, 0, sx);
  Self.SetElement(1, 1, sy);
  Self.SetElement(2, 2, sz);
  Self.SetElement(3, 3, 1);
end;

procedure TMatrix.SetRotationMatrix(const Angle: Real; const AAxis: TAxis);
{ Afhankelijk van de opgegeven as (akX, akY of akZ) wordt van de huidige
  matrix een rotatiematrix gemaakt aan de hand van de opgegeven hoek Angle. }
begin
  Self.SetIdentityMatrix;
  case AAxis of
     aX: begin
           Self.SetElement(1, 1, Cos(Angle));
           Self.SetElement(1, 2, -Sin(Angle));
           Self.SetElement(2, 1, Sin(Angle));
           Self.SetElement(2, 2, Cos(Angle));
         end;
     aY: begin
           Self.SetElement(0, 0, Cos(Angle));
           Self.SetElement(0, 2, Sin(Angle));
           Self.SetElement(2, 0, -Sin(Angle));
           Self.SetElement(2, 2, Cos(Angle));
         end;
     aZ: begin
           Self.SetElement(0, 0, Cos(Angle));
           Self.SetElement(0, 1, -Sin(Angle));
           Self.SetElement(1, 0, Sin(Angle));
           Self.SetElement(1, 1, Cos(Angle));
         end;
  end;
end;

procedure TMatrix.SetColumn(const i: integer; const AVector: TVector3d);
var j: integer;
begin
  for j := 0 to 2 do SetElement(j, i, AVector[j]);
end;

procedure TMatrix.SetColumn(const i: integer; const APoint: TPoint3d);
var j: integer;
begin
  for j := 0 to 2 do SetElement(j, i, APoint[j]);
end;

procedure TMatrix.SetColumn(const i: integer; const x, y, z: real);
begin
  SetElement(0, i, x);
  SetElement(1, i, y);
  SetElement(2, i, z);
end;

procedure TMatrix.SetRow(const i: integer; const AVector: TVector3d);
var j: integer;
begin
  for j := 0 to 2 do SetElement(i, j, AVector[j]);
end;

procedure TMatrix.SetRow(const i: integer; const APoint: TPoint3d);
var j: integer;
begin
  for j := 0 to 2 do SetElement(i, j, APoint[j]);
end;

procedure TMatrix.SetRow(const i: integer; const x, y, z: real);
begin
  SetElement(i, 0, x);
  SetElement(i, 1, y);
  SetElement(i, 2, z);
end;

procedure TMatrix.Assign(const ASourceMatrix: TMatrix);
{ Deze procedure wordt gemaakt om exactie kopin van TMatrix objecten te kunnen
  maken. }
var i, j: Integer;
begin
  for i := 0 to 3 do
    for j := 0 to 3 do
      FElements[i, j] := ASourceMatrix.Element(i, j);
end;

function TMatrix.Element(const i, j: Integer): Real;
{ Element retourneert het element op de positie (i, j) in de matrix Self. }
begin
  Element := Self.FElements[i, j];
end;

end.
