//marcus?
unit ULightMapTools;

interface

Uses
  Sysutils,GLMeshOptimizer,VectorGeometry,GLMisc, GLUtils, GLScene,
  GLWin32Viewer, GLVectorFileObjects, GLObjects,
  GLTexture, OpenGL1X, GLContext,
  GLFile3ds,GLFileOCT,GLFileMS3D,GLFileGTS,GLFileLWO,GLFileMD2,GLFileMD3,
  GLFileNMF,GLFileNurbs,GLFileObj,GLFilePLY,GLFileQ3BSP,GLFileSMD,
  GLFileSTL,GLFileTIN,
  ExtDlgs,JPEG,TGA, StdCtrls, Spin, GLNavigator, ExtCtrls, GLCadencer;

procedure ImportLMMLightMap(DestFF: TGLFreeForm; FileNameLightMapped3DS: string; DoOptimize : boolean);


implementation


function ExtractNumber(st:string): integer;
var
   t: integer;
   p,q : integer;
begin
     p := -1;
     q := -1;

     for t:= 0 to length(st) do
     begin
          if StrToIntDef(st[t],-1)>-1 then
          begin
               p := t;
               break;
          end;
     end;

     if p>-1 then
     begin
          for t:= p to length(st) do
          begin
               if StrToIntDef(st[t],-1)>-1 then
                    q := t
               else
                   Break;
          end;
     end;

     Result := StrToIntDef(copy(st,p,q-p+1),-1);
end;



procedure ImportLMMLightMap(DestFF: TGLFreeForm; FileNameLightMapped3DS: string; DoOptimize : boolean);
var
   t,u: integer;
   FF: TGLFreeForm;
   cDir : string;
   tp : TTexPoint;
begin
     // It is supposed that the passed FreeForm already loaded
     // the *Textured* 3DS file exported by LightMapMaker,
     // while this proc loads the corresponding *Lightmapped* 3DS file exported
     // by LightMapMaker to a temp FreeForm, and adds LightMaps to the passed FreeForm.
     // Thanks to Paulo Rogrio de Oliveira for the concept.


     FF := TGLFreeForm.Create(DestFF.Owner);
     FF.MaterialLibrary := DestFF.LightmapLibrary;
     FF.MaterialLibrary.Materials.Clear;
     cDir := GetCurrentDir;
     ChDir(ExtractFilePath(FileNameLightMapped3DS));
     try
        FF.LoadFromFile(FileNameLightMapped3DS);

        if DoOptimize then
        begin
             OptimizeMesh(FF.MeshObjects,[mooVertexCache, mooSortByMaterials,mooMergeObjects]);
             FF.StructureChanged;
        end;
        
        if FF.MeshObjects.Count<>DestFF.MeshObjects.Count then
           raise(Exception.Create('The file "'+FileNameLightMapped3DS+'" contains a different number of MeshObjects.'));


        for t := 0 to DestFF.MeshObjects.Count-1 do
        begin
             DestFF.MeshObjects[t].LightMapTexCoords.Clear;

             // LightmapTexture Coords
             For u := 0 to FF.MeshObjects[t].TexCoords.Count-1 do
             begin
                  tp.S := FF.MeshObjects[t].TexCoords[u][0];
                  tp.T := FF.MeshObjects[t].TexCoords[u][1];

                  DestFF.MeshObjects[t].LightMapTexCoords.Add(tp);
             end;

             // Lightmap Index
             For u := 0 to FF.MeshObjects[t].FaceGroups.Count-1 do
             begin
                  DestFF.MeshObjects[t].FaceGroups[u].LightMapIndex := ExtractNumber(FF.MeshObjects[t].FaceGroups[u].MaterialName)-1;
             end;

        end;
     finally
           ChDir(cDir);
           FF.Free;
           DestFF.StructureChanged;
     end;
end;

end.
