{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit dtmGlsTrealViewerFrm;

(*  actually just the notes from a demo
    trying to get transparency to work
{: A basic sample to demonstrate how transparency & Z-buffer work/fight together.<p>

   In this sample, only the sphere are transparent. The form has a few options
   that allow to adjust in which order objects are rendered, and what kind of
   transparency is used.<p>

   Transparency in GLScene is activated by setting Material.Blending to either
   'bmTransparency' or 'bmAdditive', AND giving a <1.0 value to the Diffuse
   color alpha channel (alpha = 0.0 means absolute transparency, alpha = 1.0
   means absolute opacity).<p>

   How do Z-Buffer & transparency work ? When point has to be rendered, OpenGL
   first checks its distance to the camera, the "Z" axis. If the distance
   check is successfull (the new point is closer), it is rendered, and if
   the point is part of a "transparent" object, then OpenGL will mix the existing
   point's color with our new point's color. If the Z check fails, OpenGL doesn't
   even bother about checking transparency.<p>

   This is why, if you want to render transparent objects, you must make sure
   you render the farthest objects first, to give transparency a chance.
   osRenderFarthestFirst
   However this effect can be usefull if you want to render mixed, half-transparent
   half-opaque objects.<p>

   They are two ways to order objects in GLScene :<ul>
   <li>ordering : can be done at design-time in the editor or at runtime with
      MoveUp/MoveDown methods
   <li>sorting : adjust the ObjectSorting property (see help for more details)
   </ul>
}
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus, ExtCtrls,
  ShellApi, {jpg,}  jpeg,
  TreeConstants, TreeVector,
  TreeCode,
  TreeModelParameters,
  VectorGeometry, {Power}
  GLScene, GLObjects,
  GLWin32Viewer, GLMisc, GLVectorFileObjects, GLTexture, GLCadencer,
  {AVIRecorder,} Buttons, GLAVIRecorder;


type
  TdtmGlsTrealViewerForm = class(TForm)
    Panel1: TPanel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    ProgressBar: TProgressBar;
    CancelButton: TButton;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    GLScene1: TGLScene;
    GLSceneViewer1: TGLSceneViewer;
    GLCamera: TGLCamera;
    GLLightSource1: TGLLightSource;
    GLDummyCube1: TGLDummyCube;
    Help2: TMenuItem;
    About1: TMenuItem;
    TreeFF: TGLFreeForm;
    Action1: TMenuItem;
    ZoomIn1: TMenuItem;
    ZoomOut1: TMenuItem;
    Cancel1: TMenuItem;
    N2: TMenuItem;
    DistanceCB: TCheckBox;
    OpenTdfDialog1: TOpenDialog;
    GLMaterialLibrary: TGLMaterialLibrary;
    FruitFF: TGLFreeForm;
    SaveDialog: TSaveDialog;
    DistanceTrackBar: TTrackBar;
    Label133: TLabel;
    Label131: TLabel;
    QualityLabel: TLabel;
    Label132: TLabel;
    VectorLabel: TLabel;
    LeavesLabel: TLabel;
    StemsLabel: TLabel;
    LeavesVLabel: TLabel;
    WindStrengthTB: TTrackBar;
    Label37: TLabel;
    WindStrengthLabel: TLabel;
    WindGustTB: TTrackBar;
    Label136: TLabel;
    WindGustLabel: TLabel;
    WindFromTB: TTrackBar;
    Label43: TLabel;
    WindFromLabel: TLabel;
    WindAnimationCB: TCheckBox;
    AnimationCB: TCheckBox;
    GLCadencer1: TGLCadencer;
    RendertoBitmap1: TMenuItem;
    Snapshot1: TMenuItem;
    N4: TMenuItem;
    Bitmap1: TMenuItem;
    Bitmapx21: TMenuItem;
    Bitmap300dpi1: TMenuItem;
    Bitmap600dpi1: TMenuItem;
    RendertoAVI1: TMenuItem;
    Rotate3601: TMenuItem;
    SkyRanger1: TMenuItem;
    WindPilot1: TMenuItem;
    FruitVLabel: TLabel;
    FruitLabel: TLabel;
    GLLightSource2: TGLLightSource;
    AVIRecorder1: TAVIRecorder;
    N5: TMenuItem;
    DisplayControls1: TMenuItem;
    SelectBackgroundColor1: TMenuItem;
    ColorDialog1: TColorDialog;
    CubeAxisVisible1: TMenuItem;
    DeRange1: TMenuItem;
    MoveUp1: TMenuItem;
    MoveDown1: TMenuItem;
    CameraDistanceLabel: TLabel;
    GLTreeSprite: TGLSprite;
    Timer1: TTimer;
    FPSLabel: TLabel;
    Label1: TLabel;
    RotationActiveCB: TCheckBox;
    StaticDistanceCB: TCheckBox;
    BigTreeRatCountLabel: TLabel;
    ShowTime: TTimer;
    DummyTree: TGLDummyCube;
    BigLeafFF: TGLFreeForm;
    SpeedButton1: TSpeedButton;
    Panel2: TPanel;
    SunHeightTB: TTrackBar;
    Label44: TLabel;
    SunHeightLabel: TLabel;
    SunStrengthTB: TTrackBar;
    Label137: TLabel;
    SunStrengthLabel: TLabel;
    SunFromTB: TTrackBar;
    Label45: TLabel;
    SunFromLabel: TLabel;
    DisplaySunControls1: TMenuItem;
    SunLightCB: TCheckBox;
    CameraLightCB: TCheckBox;
    LeafFF: TGLFreeForm;
    LightSwitcherCB: TCheckBox;
    ResetLightDefaults1: TMenuItem;
    SaveasGLSM1: TMenuItem;
    SpriteGLMaterialLibrary: TGLMaterialLibrary;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure Exit1Click(Sender: TObject);
    procedure Help1Click(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);

    procedure GLSceneViewer1MouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure GLSceneViewer1MouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure GLSceneViewer1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure About1Click(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure ZoomIn1Click(Sender: TObject);
    procedure ZoomOut1Click(Sender: TObject);
procedure LoadParameters;

    procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
      newTime: Double);
    procedure AnimationCBClick(Sender: TObject);
    procedure WindAnimationCBClick(Sender: TObject);
    procedure WindStrengthTBChange(Sender: TObject);
    procedure WindGustTBChange(Sender: TObject);
    procedure WindFromTBChange(Sender: TObject);

    procedure DistanceCBClick(Sender: TObject);
    procedure DistanceTrackBarChange(Sender: TObject);

    procedure Snapshot1Click(Sender: TObject);
    procedure Bitmap1Click(Sender: TObject);
    procedure Bitmapx21Click(Sender: TObject);
    procedure Bitmap300dpi1Click(Sender: TObject);
    procedure Bitmap600dpi1Click(Sender: TObject);

    procedure Rotate3601Click(Sender: TObject);
    procedure SkyRanger1Click(Sender: TObject);
    procedure WindPilot1Click(Sender: TObject);

procedure MakeAVIBmp;
    procedure DisplayControls1Click(Sender: TObject);
    procedure DisplaySunControls1Click(Sender: TObject);
procedure SetDisplayControls;
    procedure SunHeightTBChange(Sender: TObject);
    procedure SunStrengthTBChange(Sender: TObject);
    procedure SunFromTBChange(Sender: TObject);

    procedure SelectBackgroundColor1Click(Sender: TObject);
    procedure CubeAxisVisible1Click(Sender: TObject);
    procedure DeRange1Click(Sender: TObject);
    procedure MoveUp1Click(Sender: TObject);
    procedure MoveDown1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure RotationActiveCBClick(Sender: TObject);
    procedure ShowTimeTimer(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SunLightCBClick(Sender: TObject);
    procedure CameraLightCBClick(Sender: TObject);
    procedure ResetLightDefaults1Click(Sender: TObject);
    procedure LightSwitcherCBClick(Sender: TObject);
    procedure SaveasGLSM1Click(Sender: TObject);


  private
    { Private declarations }
    mx,my : integer;
    ReadyToWork,
    SpriteVisible,{Mouse} md:Boolean;
    AnimeChannel:Integer;
    MyTree: TTree;
    MyModelParameters: TModelParameters;
    ViewerTotalFruit, ViewerTotalFruitVertices,
    ViewerTotalLeaves,ViewerTotalLeaveVertices: integer;
    ViewerTotalStems, ViewerTotalVertices
    {,ViewerTotalFaces}: integer;
function Initialize(MaxX, MaxY, MaxZ: real; ATotalStems: integer): integer;
function AddBole(BoleSections: TList): integer;
function AddStem(Sections: TList): integer;
function AddLeaf(Points: TList; LeafColor: integer; LeafColorVariation: boolean): integer;
function AddBigLeaf(Points: TList; LeafColor: integer; LeafColorVariation: boolean): integer;
function AddFruit(Points: TList; FruitColor: integer; FruitColorVariation: boolean): integer;

    procedure RenderToBitmap(scale : Single);
    procedure AnimeMotion;
    procedure ComputeAnimeMotion;
  public
    { Public declarations }
  end;

var
  dtmGlsTrealViewerForm: TdtmGlsTrealViewerForm;

implementation

uses
  dtmGlsTrealViewBitmapFrm,       //Created
  GLGraphics, GLCrossPlatform,
  KeyBoard,
  dtmPOFvar,
  dtmGlsTrealFrm,
  dtmGlsTrealAboutFrm,           //Created
  dtmGlsTrealProgressFormUnit;  //dtmGlsTrealProgressForm.Show;

{$R *.DFM}
   var GLTreeSpriteLoaded:Boolean;
       AniSign:Integer;

procedure TdtmGlsTrealViewerForm.FormCreate(Sender: TObject);
begin
  Left:=GlsTrealViewerFormX;
  Top:=GlsTrealViewerFormY;
  GLTreeSpriteLoaded:=False;
  TreePlanted:=False; 
  AnimationActive:=False;
  DistanceActive:=False;
  WindAnimationActive:=False;
  GlsUserCancelled := False;
  {GlsViewerCancelled := False; }
  SpriteVisible  := False;
  AnimeChannel:=0;
  BigTreeRat:=0;
  BigTreeRatCount:=0;
  BigTotalLeaves:=0;
  ReadyToWork:=True;
  AniSign:=1;
  Anime360Counter:=1;
end;

procedure TdtmGlsTrealViewerForm.FormShow(Sender: TObject);
begin
  ShowTime.Enabled:=True;
end;
procedure TdtmGlsTrealViewerForm.ShowTimeTimer(Sender: TObject);
begin
  ShowTime.Enabled:=False;
  LoadParameters;
  WindAnimationCB.Checked:=WindAnimationActive;
  AnimationDistance:=1; 
  DistanceTrackBar.Position:=AnimationDistance;
end;
procedure TdtmGlsTrealViewerForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TdtmGlsTrealViewerForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  GLCadencer1.Enabled:=False;
  GlsUserCancelled := True;
  AnimationActive:=False;
  DistanceActive:=False;
  WindAnimationActive:=False;
  Application.ProcessMessages;
  GlsTrealViewerFormX:=dtmGlsTrealViewerForm.Left;
  GlsTrealViewerFormY:=dtmGlsTrealViewerForm.Top;
  If TreePlanted then
  begin
    MyTree.Destroy;{Tree data}
    MyModelParameters.Destroy;
    {Freeform data}
  end;
  TreePlanted:=False;
end;

procedure TdtmGlsTrealViewerForm.Help1Click(Sender: TObject);
begin
  ShellExecute(
    Application.Handle, // handle to parent window
    'open',             // pointer to string that specifies operation to perform
    PChar(ExtractFilePath(Application.Exename)+'tree help\glsview.htm'),{PChar(POVRayExecutable),}// pointer to filename or folder name string
    ''{PChar(POVRayCommandLine)},// pointer to string that specifies executable-file parameters
    PChar(ExtractFilePath(Application.Exename)+'tree help'),// pointer to string that specifies default directory
    SW_SHOWNORMAL);
end;

procedure TdtmGlsTrealViewerForm.About1Click(Sender: TObject);
begin
  dtmGlsTrealAboutForm:=TdtmGlsTrealAboutForm.Create(Application);
  try
    dtmGlsTrealAboutForm.ShowModal;
  finally
    dtmGlsTrealAboutForm.Release;
  end;
end;

procedure TdtmGlsTrealViewerForm.CubeAxisVisible1Click(Sender: TObject);
begin
  CubeAxisVisible1.Checked:= (not CubeAxisVisible1.Checked);
  GLDummyCube1.VisibleAtRunTime:=CubeAxisVisible1.Checked;
  GLDummyCube1.ShowAxes:=CubeAxisVisible1.Checked;
end;

procedure TdtmGlsTrealViewerForm.DisplayControls1Click(Sender: TObject);
begin
  DisplayControls1.Checked:= (not DisplayControls1.Checked);
  SetDisplayControls;
end;
procedure TdtmGlsTrealViewerForm.DisplaySunControls1Click(Sender: TObject);
begin
  DisplaySunControls1.Checked:= (not DisplaySunControls1.Checked);
  SetDisplayControls;
end;
procedure TdtmGlsTrealViewerForm.SetDisplayControls;
Begin
  If DisplayControls1.Checked then
  begin
    Panel1.Visible:=True;
    Panel1.Align:=alBottom;
{    GLSceneViewer1.Align:=alnone;}         
    Panel2.Visible:=DisplaySunControls1.Checked;
    If  DisplaySunControls1.Checked then
      Panel2.Align:=alBottom;
  end else
  begin
    {GLSceneViewer1.Align:=alnone; }
    Panel2.Visible:=False;
    Panel2.Align:=alnone;
    Panel1.Visible:=False;
    Panel1.Align:=alnone;
    DisplaySunControls1.Checked:=False;
  end;
  GLSceneViewer1.Align:=alClient;
End;

//-----------------------------------------------------
//-----------------------------------------------------

procedure TdtmGlsTrealViewerForm.SunHeightTBChange(Sender: TObject);
begin
  SunHeightLabel.Caption:=Inttostr(SunHeightTB.Position);
  If LightSwitcherCB.Checked then
  GLLightSource2.Position.y:=SunHeightTB.Position
  else GLLightSource1.Position.y:=SunHeightTB.Position;
  {FModelParameters.SunHeight;}
end;

procedure TdtmGlsTrealViewerForm.SunStrengthTBChange(Sender: TObject);
begin
  SunStrengthLabel.Caption:=Inttostr(SunStrengthTB.Position);
  If LightSwitcherCB.Checked then
  GLLightSource2.ConstAttenuation:=(SunStrengthTB.Position/100)
  else   GLLightSource1.ConstAttenuation:=(SunStrengthTB.Position/100);
  {FModelParameters.SunStrength;}
end;

procedure TdtmGlsTrealViewerForm.SunFromTBChange(Sender: TObject);
begin
  SunFromLabel.Caption:=Inttostr(SunFromTB.Position);
  If LightSwitcherCB.Checked then
  GLLightSource2.Position.x:=180-SunFromTB.Position
  else GLLightSource1.Position.x:=180-SunFromTB.Position;
    {FModelParameters.SunFrom;}
end;
procedure TdtmGlsTrealViewerForm.SunLightCBClick(Sender: TObject);
begin
  GLLightSource2.Shining:=SunLightCB.Checked;
end;

procedure TdtmGlsTrealViewerForm.CameraLightCBClick(Sender: TObject);
begin
  GLLightSource1.Shining:=CameraLightCB.Checked;
end;
procedure TdtmGlsTrealViewerForm.LightSwitcherCBClick(Sender: TObject);
begin
{}
end;
procedure TdtmGlsTrealViewerForm.ResetLightDefaults1Click(Sender: TObject);
begin
 SunHeightTB.Position:= 90;
 SunStrengthTB.Position:= 100;
 SunFromTB.Position:= 90;
 GLLightSource1.Position.x:= 0;
 GLLightSource1.ConstAttenuation:= 1;
 GLLightSource1.Position.y:= 0;

 GLLightSource2.Position.x:= 0;
 GLLightSource2.ConstAttenuation:= 1;
 GLLightSource2.Position.y:= 100;

 LightSwitcherCB.Checked:=True;{FALSE > Light 1 : Camera Active}
 SunLightCB.Checked:=True;
 CameraLightCB.Checked:=True;
 GLLightSource2.Shining:=SunLightCB.Checked;
 GLLightSource1.Shining:=CameraLightCB.Checked;
end;
//-----------------------------------------------------
//-----------------------------------------------------

procedure TdtmGlsTrealViewerForm.GLSceneViewer1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mx:=X; my:=Y;
  md:=True;
end;

procedure TdtmGlsTrealViewerForm.GLSceneViewer1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
   d : Single;
begin
   if md and (ssLeft in Shift) then begin
      {if ssLeft in Shift then begin }

      if ssCtrl  in Shift then begin
         d:=GLCamera.DistanceToTarget*0.01*(x-mx+y-my);
         if IsKeyDown('x') then
            DummyTree{TreeFreeForm}.Translate(d, 0, 0)
         else if IsKeyDown('y') then
            DummyTree{TreeFreeForm}.Translate(0, d, 0)
         else if IsKeyDown('z') then
            DummyTree{TreeFreeForm}.Translate(0, 0, d)
         else GLCamera.RotateObject(TreeFF, my-y, mx-x);
      end else
      GLCamera.MoveAroundTarget(my-y, mx-x);
      CameraDistanceLabel.Caption:=Inttostr(Round(GLCamera.DistanceToTarget));
      mx:=x; my:=y;
   end;
end;

procedure TdtmGlsTrealViewerForm.MoveUp1Click(Sender: TObject);
begin
  DummyTree{TreeFreeForm}.Translate(0, 1, 0)
end;
procedure TdtmGlsTrealViewerForm.MoveDown1Click(Sender: TObject);
begin
  DummyTree{TreeFreeForm}.Translate(0, -1,0)
end;
procedure TdtmGlsTrealViewerForm.GLSceneViewer1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   md:=False;
end;

procedure TdtmGlsTrealViewerForm.FormMouseWheel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
begin
   if TreeFF.MeshObjects.Count>0 then
   begin
      GLCamera.AdjustDistanceToTarget(Power(1.05, WheelDelta/120));
      GLCamera.DepthOfView:=2*GLCamera.DistanceToTarget
                            +2*TreeFF.BoundingSphereRadius;
      CameraDistanceLabel.Caption:=Inttostr(Round(GLCamera.DistanceToTarget));
      If DistanceActive then
      DistanceTrackBar.Position:=Round(GLCamera.DistanceToTarget);
   end;
   Handled:=True;
end;

procedure TdtmGlsTrealViewerForm.ZoomIn1Click(Sender: TObject);
var
   h : Boolean;
begin
   FormMouseWheel(Self, [], -120, Point(0, 0), h);
end;

procedure TdtmGlsTrealViewerForm.ZoomOut1Click(Sender: TObject);
var
   h : Boolean;
begin
   FormMouseWheel(Self, [], 120, Point(0, 0), h);
end;



{   A utility function, this takes the bitmap and uses Form2 to display it with
   a regular TImage component.}
procedure ViewBitmap(aBitmap : TBitmap; caption : String);
begin
  dtmGlsTrealViewBitmapForm:=TdtmGlsTrealViewBitmapForm.Create(Application);
   dtmGlsTrealViewBitmapForm.Image1.Picture.Bitmap:=aBitmap;
   dtmGlsTrealViewBitmapForm.Image1.Width:=aBitmap.Width;
   dtmGlsTrealViewBitmapForm.Image1.Height:=aBitmap.Height;
   dtmGlsTrealViewBitmapForm.Caption:=caption;
   if (aBitmap.Width<Screen.Width) and (aBitmap.Height<Screen.Height) then begin
      dtmGlsTrealViewBitmapForm.Width:=aBitmap.Width;
      dtmGlsTrealViewBitmapForm.Height:=aBitmap.Height;
   end else begin
      dtmGlsTrealViewBitmapForm.Width:=Round(Screen.Width*0.75);
      dtmGlsTrealViewBitmapForm.Height:=Round(Screen.Height*0.75);
   end;
  try
    dtmGlsTrealViewBitmapForm.ShowModal;
  finally
    dtmGlsTrealViewBitmapForm.Release;
  end;
end;

procedure TdtmGlsTrealViewerForm.SelectBackgroundColor1Click(Sender: TObject);
begin
  ColorDialog1.Color:=GLSceneViewer1.Buffer.BackGroundColor;
  If ColorDialog1.Execute then
  GLSceneViewer1.Buffer.BackGroundColor:= ColorDialog1.Color;
  {GLSceneViewer1.Buffer.BackGroundColor:=clBtnFace;}
end;

procedure TdtmGlsTrealViewerForm.RenderToBitmap(scale : Single);
var
   bmp : TBitmap;
   pt : Int64;
   delta : Double;
begin
   pt:=StartPrecisionTimer;
   // Rendering to a bitmap requires an existing bitmap,
   // so we create and size a new one
   bmp:=TBitmap.Create;
   // Don't forget to specify a PixelFormat, or current screen pixel format
   // will be used, which may not suit your purposes!
   bmp.PixelFormat:=pf24bit;
   bmp.Width:=Round(GLSceneViewer1.Width*scale);
   bmp.Height:=Round(GLSceneViewer1.Height*scale);
   // Here we just request a render
   // The second parameter specifies DPI (Dots Per Inch), which is
   // linked to the bitmap's scaling
   // "96" is the "magic" DPI scale of the screen under windows
   GLSceneViewer1.Buffer.RenderToBitmap(bmp, Round(96*scale));
   delta:=StopPrecisionTimer(pt);
   ViewBitmap(bmp, Format('RenderToBitmap %dx%d - %.1f ms',
                          [bmp.Width, bmp.Height, delta*1000]));
   bmp.Free;
end;


procedure TdtmGlsTrealViewerForm.Snapshot1Click(Sender: TObject);
var
   bmp32 : TGLBitmap32;
   bmp : TBitmap;
   pt : Int64;
   delta : Double;
begin
   pt:=StartPrecisionTimer;
   // CreateSnapShot returns a TGLBitmap32, which is a low-level data buffer.
   // However TGLBitmap32 can spawn a regular TBitmap, which we use here
   bmp32:=GLSceneViewer1.Buffer.CreateSnapShot;
   bmp:=bmp32.Create32BitsBitmap;
   delta:=StopPrecisionTimer(pt);
   // Display the bitmap for the user to see and gaze in everlasting awe...
   ViewBitmap(bmp, Format('SnapShot %dx%d - %.3f ms',
                          [bmp.Width, bmp.Height, delta*1000]));
   // Don't forget to free your TGLBitmap32 and TBitmap!
   bmp.Free;
   bmp32.Free;
end;

procedure TdtmGlsTrealViewerForm.Bitmap1Click(Sender: TObject);
begin
   // Render at viewer resolution (scale = 1, DPI = 96)
   RenderToBitmap(1);
end;

procedure TdtmGlsTrealViewerForm.Bitmapx21Click(Sender: TObject);
begin
   // Render at twice viewer resolution (scale = 2, DPI = 192 = 96x2)
   RenderToBitmap(2);
end;

procedure TdtmGlsTrealViewerForm.Bitmap300dpi1Click(Sender: TObject);
begin
   // Screen is "magic" 96 dpi, this gives us our scale
   RenderToBitmap(300/96);
end;

procedure TdtmGlsTrealViewerForm.Bitmap600dpi1Click(Sender: TObject);
begin
   // Screen is "magic" 96 dpi, this gives us our scale
   RenderToBitmap(600/96);
end;


procedure TdtmGlsTrealViewerForm.Timer1Timer(Sender: TObject);
begin
   FPSLabel.Caption:=Format('%.2f FPS', [GLSceneViewer1.FramesPerSecond]);
   GLSceneViewer1.ResetPerformanceMonitor;
end;

procedure TdtmGlsTrealViewerForm.GLCadencer1Progress(Sender: TObject;
  const deltaTime, newTime: Double);
var    h : Boolean;
Procedure ReverseCSign;
begin
  If AniSign =1 then AniSign := -1 else AniSign := 1;
end;
begin
  Anime360Counter:=Anime360Counter+AniSign;
  If (Anime360Counter > 360) then ReverseCSign else
  If (Anime360Counter <1 ) then ReverseCSign ;
  If DistanceActive then
  begin
    If (not StaticDistanceCB.Checked) then
    FormMouseWheel(Self, [], (20*AniSign), Point(0, 0), h);
  end;
  If RotationActive then
  GLCamera.RotateObject(TreeFF, 0,1);
  If WindAnimationActive then ;
  If AnimationCB.Checked then  ComputeAnimeMotion;
end;

procedure TdtmGlsTrealViewerForm.CancelButtonClick(Sender: TObject);
begin
{  if MessageDlg('Are you sure?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then }
  GlsUserCancelled := True;
  {GlsViewerCancelled := True; }
  AnimationCB.Checked:=False;
  GLCadencer1.Enabled:= AnimationCB.Checked;
  Timer1.Enabled:= AnimationCB.Checked;
  AnimationActive:= AnimationCB.Checked;
  DistanceCB.Checked:= AnimationCB.Checked;
  DistanceActive:= DistanceCB.Checked;
  WindAnimationCB.Checked:= AnimationCB.Checked;
  WindAnimationActive:= WindAnimationCB.Checked;
  RotationActiveCB.Checked:= AnimationCB.Checked;
  RotationActive:= RotationActiveCB.Checked;
end;

procedure TdtmGlsTrealViewerForm.AnimationCBClick(Sender: TObject);
begin
  Anime360Counter:=1;
  AnimationDistance:=0;
  DistanceTrackBar.Position:=0;
  GLCadencer1.Enabled:= AnimationCB.Checked;
  Timer1.Enabled:= AnimationCB.Checked;
  AnimationActive:= AnimationCB.Checked;
  If AnimationCB.Checked then
  begin      {Done in Cadencer}

  end else
  begin  {Turn them all OFF}
    DistanceCB.Checked:= AnimationCB.Checked;
      DistanceActive:= DistanceCB.Checked;
      StaticDistanceCB.Checked:= AnimationCB.Checked;
    WindAnimationCB.Checked:= AnimationCB.Checked;
      WindAnimationActive:= WindAnimationCB.Checked;
    RotationActiveCB.Checked:= AnimationCB.Checked;
      RotationActive:= RotationActiveCB.Checked;
  end;
end;
procedure TdtmGlsTrealViewerForm.RotationActiveCBClick(Sender: TObject);
begin
  RotationActive:= RotationActiveCB.Checked;
end;

procedure TdtmGlsTrealViewerForm.WindAnimationCBClick(Sender: TObject);
begin  {Wind changes with TIME per parameters}
  WindAnimationActive:= WindAnimationCB.Checked;
  {When it does Anything... NONE do anything as of NOW
  AVI does ?   per Time
  Cadencer does ?    ""
  Manual movement does ?  stepped changes... }
end;
procedure TdtmGlsTrealViewerForm.WindStrengthTBChange(Sender: TObject);
begin
  WindStrengthLabel.Caption:=Inttostr(WindStrengthTB.Position);
  MyModelParameters.WindStrength:= WindStrengthTB.Position;
  ComputeAnimeMotion;
end;
procedure TdtmGlsTrealViewerForm.WindGustTBChange(Sender: TObject);
begin
  WindGustLabel.Caption:=Inttostr(WindGustTB.Position);
  MyModelParameters.WindGust:= WindGustTB.Position;
  ComputeAnimeMotion;
end;
procedure TdtmGlsTrealViewerForm.WindFromTBChange(Sender: TObject);
begin
  WindFromLabel.Caption:=Inttostr(WindFromTB.Position);
  MyModelParameters.WindFrom:= WindFromTB.Position;
  ComputeAnimeMotion;
end;


procedure TdtmGlsTrealViewerForm.DistanceCBClick(Sender: TObject);
begin
  DistanceActive:= DistanceCB.Checked;
end;


procedure TdtmGlsTrealViewerForm.SpeedButton1Click(Sender: TObject);
begin
  GLTreeSprite.Visible := (not GLTreeSprite.Visible);
       If TreeFF.MeshObjects.Count>0 then TreeFF.Visible:=(not GLTreeSprite.Visible);
       If MyTree.TotalGrownLeaves>0 then LeafFF.Visible:=(not GLTreeSprite.Visible);
       If BigTreeRatCount>0 then BigLeafFF.Visible:=(not GLTreeSprite.Visible);
       If MyTree.TotalFruit>0 then FruitFF.Visible:=(not GLTreeSprite.Visible);
end;

{The actual Degradation Manually or Cadencer..AVI Changes this}
{Get the Range and # of Objects; Divide into groups... NOT just 0..total
Visible..Larger objects ..
Sprite available ?..for now DESIGN is for this to be a Sprite Class
Or NOT ... Sprite 'shade - color' is always wrong}
{Really only need Object at range Closer than ~ 100}
procedure TdtmGlsTrealViewerForm.DistanceTrackBarChange(Sender: TObject);
var
{  Ranges,}step, Stepper, iii:Integer;
   mesh : TBaseMeshObject;
begin
  If ReadyToWork then begin  {No changes during work}
  ReadyToWork:=False;
         {Minimum change distance}
  If (BigLeafOffSet < DistanceTrackBar.Position)then
  begin

  {ComputedDistanceLabel.Caption:= Inttostr(Round(GLCamera.DistanceToTarget));}
  Stepper:=(DistanceTrackBar.Position-BigLeafOffSet) div 10;
  If AnimationDistance < DistanceTrackBar.Position then
  begin {decrease Leafs.. going farther away}
    {Divide and Conquer}
    If  (DistanceTrackBar.Position> (BigLeafOffSet +200)) then
    begin
      If (not SpriteVisible) then
      begin
        SpriteVisible:=True;
        If GLTreeSpriteLoaded then
        begin
          GLTreeSprite.Visible:=True;
          If TreeFF.MeshObjects.Count>0 then TreeFF.Visible:=False;
          If BigTreeRatCount>0 then BigLeafFF.Visible:=False;
        end;
        If MyTree.TotalGrownLeaves>0 then LeafFF.Visible:=False;
        If MyTree.TotalFruit>0 then FruitFF.Visible:=False;
      end;
    end else
    begin
    {BigTreeRatCount Opposite On..Off the Leaf and Fruit}
    If BigTreeRatCount>0 then
    begin
      step:=1;
      for iii:=0 to (BigLeafFF.MeshObjects.Count-1) do
      begin
        If ((step < stepper)) then
        begin
          mesh:=BigLeafFF.MeshObjects.Items[iii];
          mesh.Visible:=True;
        end;
        inc(step);
        If step>20 then step:=1;
      end;
      BigLeafFF.StructureChanged;
    end;
    If MyTree.TotalGrownLeaves>0 then
    begin
      step:=1;
      for iii:=0 to (LeafFF.MeshObjects.Count-1) do
      begin
        If ((step < stepper)) then
        begin
          mesh:=LeafFF.MeshObjects.Items[iii];
          mesh.Visible:=False;
        end;
        inc(step);
        If step>20 then step:=1;
      end;
      LeafFF.StructureChanged;
    end;
    If MyTree.TotalFruit>0 then
    begin
      step:=1;
      for iii:=0 to (FruitFF.MeshObjects.Count-1) do
      begin
        If ((step < stepper)) then
        begin
          mesh:=FruitFF.MeshObjects.Items[iii];
          mesh.Visible:=False;
        end;
        inc(step);
        If step>20 then step:=1;
      end;
      FruitFF.StructureChanged;
    end;
    end;
  end else
  begin  {AnimationDistance > QualityTrackBar.Position}
    If  ((DistanceTrackBar.Position< (BigLeafOffSet+201))
         and (SpriteVisible)) then
    begin
      SpriteVisible:=False;
      If GLTreeSpriteLoaded then
      begin
        GLTreeSprite.Visible:=False;
        If TreeFF.MeshObjects.Count>0 then TreeFF.Visible:=True;
        If BigTreeRatCount>0 then BigLeafFF.Visible:=True;
      end;
      If MyTree.TotalGrownLeaves>0 then LeafFF.Visible:=True;
      If MyTree.TotalFruit>0 then FruitFF.Visible:=True;
    end else
    begin
    {BigTreeRatCount Opposite On..Off the Leaf and Fruit}
    If BigTreeRatCount>0 then
    begin
      step:=1;
      for iii:=0 to (BigLeafFF.MeshObjects.Count-1) do
      begin
        If ((step > stepper)) then
        begin
          mesh:=BigLeafFF.MeshObjects.Items[iii];
          mesh.Visible:=False;
        end;
        inc(step);
        If step>20 then step:=1;
      end;
      BigLeafFF.StructureChanged;
    end;
    If MyTree.TotalGrownLeaves>0 then
    begin
      step:=1;
      for iii:=0 to (LeafFF.MeshObjects.Count-1) do
      begin
        If ((step > stepper)) then
        begin
          mesh:=LeafFF.MeshObjects.Items[iii];
          mesh.Visible:=True;
        end;
        inc(step);
        If step>20 then step:=1;
      end;
      LeafFF.StructureChanged;
    end;
    If MyTree.TotalFruit>0 then
    begin
      step:=1;
      for iii:=0 to (FruitFF.MeshObjects.Count-1) do
      begin
        If ((step > stepper)) then
        begin
          mesh:=FruitFF.MeshObjects.Items[iii];
          mesh.Visible:=True;
        end;
        inc(step);
        If step>20 then step:=1;
      end;
      FruitFF.StructureChanged;
    end;
    end;
    end;
  end;
  QualityLabel.Caption:=Inttostr(DistanceTrackBar.Position);
  AnimationDistance := DistanceTrackBar.Position;
  ReadyToWork:=True;
  end;
end;


procedure TdtmGlsTrealViewerForm.Rotate3601Click(Sender: TObject);
begin{Record AVI while rotating Cube-Tree}
  AnimeChannel:=1;
  AnimationCB.Checked:=False;
  GLCadencer1.Enabled:=False;
  MakeAVIBmp;
end;

procedure TdtmGlsTrealViewerForm.DeRange1Click(Sender: TObject);
begin
  {Record AVI while Range Degrading Cube-Tree into Distance}
  AnimeChannel:=2;
  AnimationCB.Checked:=False;
  GLCadencer1.Enabled:=False;
  MakeAVIBmp;
end;

procedure TdtmGlsTrealViewerForm.SkyRanger1Click(Sender: TObject);
begin
  {Record AVI while Range Degrading Cube-Tree IN PLACE}
  AnimeChannel:=3;
  AnimationCB.Checked:=False;
  GLCadencer1.Enabled:=False;
  MakeAVIBmp;
end;

procedure TdtmGlsTrealViewerForm.WindPilot1Click(Sender: TObject);
begin
  {Record AVI while Blowing Wind up the Cube-Tree}
  AnimeChannel:=4;
  AnimationCB.Checked:=False;
  GLCadencer1.Enabled:=False;
  MakeAVIBmp;
end;

procedure TdtmGlsTrealViewerForm.AnimeMotion;
var    h : Boolean;
Begin
  Case AnimeChannel of
  1: {rotating Cube-Tree }
    GLCamera.RotateObject(DummyTree{TreeFreeForm}, 0,1);
  2:{Range Degrading Camera .. Cube-Tree : Tree gets smaller}
     {Change the Camera to Tree distance: Move Camera}
     FormMouseWheel(Self, [], 20, Point(0, 0), h);
  3:;{Range Degrading Cube-Tree in place : Make tree degraded at same location}
     {Static View... Change the Tree degradation}
  4:;{Blowing Wind up the Cube-Tree}{Do with Anime360Counter changing time}
  end;
  {Compute NEW TREE}
  ComputeAnimeMotion;
End;

procedure TdtmGlsTrealViewerForm.ComputeAnimeMotion;
Begin {Given current parameters Recompute and Display Tree}
  {LoadParameters;}
  If StaticDistanceCB.Checked{DistanceActive} then
  Begin
    DistanceTrackBar.Position:=((2000 div 360)*Anime360Counter);
  End;
  If WindAnimationActive then
  Begin
    {Recompute TREE ... all changes due to structure movement}
    {WindAnimationActive
     WindStrengthTB.Position
     WindGustTB.Position
     WindFromTB.Position}
  End;
  CameraDistanceLabel.Caption:=Inttostr(Round(GLCamera.DistanceToTarget));
End;

procedure TdtmGlsTrealViewerForm.MakeAVIBmp;
var SavedCap : string;
begin
   // if AVIRecorder1.filename is empty, a dialog box will appear asking
   // for the filename. CreateAVIFile() will return a bool
   // indicating if user presses "cancel" in the dialog box.
  AVIRecorder1.Height:=GLSceneViewer1.Height;
  AVIRecorder1.Width:=GLSceneViewer1.Width;   
  if not AVIRecorder1.CreateAVIFile then Exit;
  SavedCap:=dtmGlsTrealViewerForm.caption;
  GlsUserCancelled := False;
  {GlsViewerCancelled := False; }
  dtmGlsTrealViewerForm.caption:='Press Cancel Button to abort';
  ProgressBar.Position:=0;
  ProgressBar.Max:=100;
  Anime360Counter:=1;
  Application.ProcessMessages;
  try
    while (Anime360Counter<361)  do
    begin
      If GlsUserCancelled then Anime360Counter:=370;
      {Create the Changed Tree}
      AnimeMotion;
      ProgressBar.Max:=100; {Tree changes the Max and Position}
      ProgressBar.Position:=round((Anime360Counter/360)*100);
      dtmGlsTrealViewerForm.caption:='Press Cancel Button to abort '+
          Inttostr(ProgressBar.Position)+ ' Percent done';
       // you might want to update your progress bar here.
      Application.ProcessMessages; // so that our app. is not freezed,
                                      // and will accept user abort.
      AVIRecorder1.AddAVIFrame;
      inc(Anime360Counter);
    end;
  finally
    // if GlsViewerCancelled, CloseAVIFile will
    // also delete the unfinished file.
     AVIRecorder1.CloseAVIFile(GlsUserCancelled); // if UserAbort, CloseAVIFile will
                                            // also delete the unfinished file.
     dtmGlsTrealViewerForm.caption:=SavedCap;
     ProgressBar.Position:=0;
   end;
End;



procedure TdtmGlsTrealViewerForm.LoadParameters;
var
  h : Boolean;
  StemSize, LeafSize, FruitSize,
  FileIncrementer, iii: integer;
  mesh : TBaseMeshObject;
  function ExportTreeMesh(AStem: TStem): integer;
  { Pre: AStem <> nil
    Ret: 0, ExportTreeMesh was not successful
         1, ExportTreeMesh was successful, the tree mesh was successfully
            processed by the selected plugin }
  var i: integer;
  begin
    with AStem do
    begin
      Application.ProcessMessages;
      if AddBole(BoleSections) = 0 then
      begin
        Result := 0;
        Exit;
      end;
      if AddStem(Sections) = 0 then
      begin
        Result := 0;
        Exit;
      end;
      Result := 1;

      for i := 0 to SubStems.Count - 1 do
      begin
        if ExportTreeMesh(TStem(SubStems[i])) = 0 then
        begin
          Result := 0;
          Exit;
        end;
        Result := 1;
      end;
FruitShapeRotator:=0;
      for i := 0 to Fruit.Count - 1 do
      begin
        if AddFruit(Fruit[i], MyModelParameters.FruitColor, MyModelParameters.FruitColorVariation) = 0 then
        begin
          Result := 0;
          Exit;
        end;
        Result := 1;
      end;
LeafShapeRotator:=0;
      for i := 0 to Leafs.Count - 1 do
      begin
        if AddLeaf(Leafs[i], MyModelParameters.LeafColor, MyModelParameters.LeafColorVariation) = 0 then
        begin
          Result := 0;
          Exit;
        end;
        Result := 1;
      end;

      for i := 0 to BigLeafs.Count - 1 do
      begin
        if AddBigLeaf(BigLeafs[i], MyModelParameters.LeafColor, MyModelParameters.LeafColorVariation) = 0 then
        begin
          Result := 0;
          Exit;
        end;
        Result := 1;
      end;
    end;
  end;
begin
  If TreePlanted then
  {Destroy stuff so NOT doubled when loading a different tree}
  begin    {But it crashes .. and or doubled anyway}
    MyTree.Destroy;{Tree data}
    {Freeform data}
    TreeFF.Visible:=False;
    LeafFF.Visible:=False;
    BigLeafFF.Visible:=False;
    FruitFF.Visible:=False;
    for iii:=TreeFF.MeshObjects.Count-1 downto 1 do
     TreeFF.MeshObjects.free;
    for iii:=LeafFF.MeshObjects.Count-1 downto 1 do
     LeafFF.MeshObjects.free;
    for iii:=BigLeafFF.MeshObjects.Count-1 downto 1 do
     BigLeafFF.MeshObjects.free;
    for iii:=FruitFF.MeshObjects.Count-1 downto 1 do
     FruitFF.MeshObjects.free;
    {Do not destroy.. not created programmatically}
  end;
   dtmGlsTrealViewerForm.Cursor := crHourGlass;
  { Initialize the model parameters class }
  with dtmGlsTrealForm do
  begin
{showmessage('ok 0');}
    LevelsNumberEditExit(nil);
    { The last level of recursion is reserved for leaves, so we only create
      LevelsNumberEdit.GetValue - 1 stem parameter levels. 0..Level}
     { showmessage(inttostr(LeafTextureCB.Items.Count));}
    MyModelParameters := TModelParameters.Create(GlobalLevelsNumber,
                          LeafTextureCB.Items.Count,
                          LeafShapeCB.Items.Count,
                          FruitTextureCB.Items.Count,
                          FruitShapeCB.Items.Count);
    if GlobalLevelsNumber= 0 then
    begin
      MessageDlg('Levels should be a value greater than zero!', mtError, [mbOK], 0);
      Exit;
    end;
    InitModelParameters(MyModelParameters);
    { Initialize the random number generator }
    if RandomTreeSeedNumber = 0 then Randomize
    else RandSeed := RandomTreeSeedNumber;
  end;
{showmessage('ok 01');}
   if Random(2) = 0 then WindSign := -1 else WindSign := 1;

  //load the materials
    If FileExists(GlsDataPath+ MyModelParameters.TreeBarkFile) then
    Begin
      {GLMaterialLibrary.AddTextureMaterial('TreeBark', FModelParameters.TreeBarkFile)}
      GLMaterialLibrary.Materials[0].Material.Texture.Image.LoadFromFile(GlsDataPath+ MyModelParameters.TreeBarkFile);
      TreeFF.Material.MaterialLibrary:=GLMaterialLibrary;
      TreeFF.Material.LibMaterialName:='TreeBark';
    end
    else
    begin
      TreeFF.Material.LibMaterialName:='';
      MyModelParameters.TreeBarkFile := '';
     {Set colors.. does not work...If Textured   Emission  Ambient}
     TreeFF.Material.FrontProperties.Ambient.AsWinColor:=
              MyModelParameters.TreeColor;
    end;


    FileIncrementer:=1;
    If MyModelParameters.FruitTextureCount > 0 then
    begin
      FruitFF.Material.blendingmode := bmTransparency;
      FruitFF.Material.MaterialLibrary:=GLMaterialLibrary;
      FruitFF.Material.LibMaterialName:='FruitTexture0';
      For iii:=0 to MyModelParameters.FruitTextureCount-1 do
      begin
        If FileExists(GlsDataPath+ MyModelParameters.FruitTextureFile[iii]) then
        Begin
          inc(FileIncrementer);
          GLMaterialLibrary.AddTextureMaterial('FruitTexture'+inttostr(iii), GlsDataPath+MyModelParameters.FruitTextureFile[iii]);
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.blendingmode := bmTransparency;
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.Texture.texturemode   := tmModulate;
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.Texture.imageAlpha    := tiaSuperBlackTransparent;
          {GLMaterialLibrary.Materials[FileIncrementer+iii].Material.FrontProperties.Diffuse.Alpha:=0.5;
           GLMaterialLibrary.Materials[FileIncrementer+iii].Material.BackProperties.Diffuse.Alpha:=0.3;}
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.Texture.Disabled   := False;
          {Hierarchy covers this FruitFF.ObjectsSorting := osRenderBlendedLast;}
        end;
      end;
    end
    else
    begin
      FruitFF.Material.LibMaterialName:='';
      FruitFF.Material.FrontProperties.Ambient.AsWinColor:=
              MyModelParameters.FruitColor;
    end;

{showmessage('ok 02');}
    If MyModelParameters.LeafTextureCount > 0 then
    begin
      BigLeafFF.Material.blendingmode := bmTransparency;
      LeafFF.Material.blendingmode := bmTransparency;
      LeafFF.Material.MaterialLibrary:=GLMaterialLibrary;
      {this used .. changes per leaf created when made}
      LeafFF.Material.LibMaterialName:='LeafTexture0';

      For iii:=0 to MyModelParameters.LeafTextureCount-1 do
      begin
        If FileExists(GlsDataPath+ MyModelParameters.LeafTextureFile[iii]) then
        Begin
          inc(FileIncrementer);
          GLMaterialLibrary.AddTextureMaterial('LeafTexture'+inttostr(iii), GlsDataPath+MyModelParameters.LeafTextureFile[iii]);
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.blendingmode := bmTransparency;
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.Texture.texturemode   := tmModulate;
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.Texture.imageAlpha    := tiaSuperBlackTransparent;
          {GLMaterialLibrary.Materials[FileIncrementer+iii].Material.FrontProperties.Diffuse.Alpha:=0.5;
           GLMaterialLibrary.Materials[FileIncrementer+iii].Material.BackProperties.Diffuse.Alpha:=0.3;}
          GLMaterialLibrary.Materials[FileIncrementer+iii].Material.Texture.Disabled   := False;
          {Hierarchy covers this LeafFF.ObjectsSorting := osRenderBlendedLast;}
        end;
      end;
    end
    else
    begin
      LeafFF.Material.LibMaterialName:='';
      LeafFF.Material.FrontProperties.Ambient.AsWinColor:=
             MyModelParameters.LeafColor;
      LeafFF.Material.BackProperties.Ambient.AsWinColor:=
             MyModelParameters.LeafBackColor;
      {LeafFF.Material.FrontProperties.Emission.Alpha:=0.5;}
      {LeafFF.Material.FrontProperties.Diffuse.Alpha:=0.3;
       LeafFF.Material.BackProperties.Diffuse.Alpha:=0.3;}
    end;
{showmessage('ok 3');}
{Showmessage(inttostr(FModelParameters.LeafTextureCount)+' : '+FModelParameters.LeafTextureFile[0]+
inttostr(FModelParameters.LeafShapeCount)+' : '+FModelParameters.LeafShapeFile[0]+ #13#10+
inttostr(FModelParameters.FruitTextureCount)+' : '+FModelParameters.FruitTextureFile[0]+
inttostr(FModelParameters.FruitShapeCount)+FModelParameters.FruitShapeFile[0]);}

   {Parameters Initialized
    FTree :=TTree.Create, FTree.Trunk.CreateStructure, FTree.Grow}
  GlsUserCancelled := False;
  {GlsViewerCancelled := False; }
  dtmGlsTrealProgressForm.Show;
  dtmGlsTrealProgressForm.Cursor := crHourGlass;
  dtmGlsTrealProgressForm.Update;
  { Create the entire tree structure }
LeafShapeRotator:=0;
FruitShapeRotator:=0;
  MyTree := TTree.Create(MyModelParameters,MyModelParameters.Season);
{ FTree := TTree.CreateBole(FModelParameters,FModelParameters.Season);}

  MyTree.Trunk.CreateStructure(nil, MyTree.TrunkLength, 0, 0);
{For i:= 0 to FModelParameters.BoleSplits -1 do
 FTree.Trunk[i].CreateBoleStructure(nil, FTree.TrunkLength, 0, 0);}

  dtmGlsTrealProgressForm.ProgressBar.Max := MyTree.TotalStems  +MyTree.TotalGrownLeaves+ MyTree.TotalFruit - 1;
  dtmGlsTrealProgressForm.ProgressBar.Position := 0;

    BigTreeRat:=(MyTree.TotalGrownLeaves div BigLeavesToMake);
    BigTotalLeaves:=0;
  { And create the tree mesh points }
  TreePlanted:=True;{This tells form to delete
                     AND signals Progress bar use in GLSTree}
  {TEST the Memory available > the Memory Required}
  StemSize:=50{Average vertices}*5{CurveRes}*100{Points};
  LeafSize:=10{Average shape points}*100{Points};
  FruitSize:=10{Average shape points}*100{Points};
  If (     {Size of each Object ? using 1000 too small ?}
  {Point=24 bytes}
      ((MyTree.TotalStems *StemSize)
       + ((BigLeavesToMake
          +MyTree.TotalGrownLeaves)*LeafSize)
       + (MyTree.TotalFruit *FruitSize))
     > MemoryCapacity) then
   begin
     If MessageDlg('Not Enough Memory.  Exit now?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
     begin
       Close;
     end;
   end;
{showmessage('ok 4'); }
  MyTree.Grow; { FTree.GrowTreeBole;}
  { Close the progress form }
  dtmGlsTrealProgressForm.Close;

   BigTotalLeaves:=0;
    If FileExists(GlsDataPath+ MyModelParameters.TreeSpriteFile) then
    Begin
      GLTreeSprite.Width:=   Round(MyTree.MaxY{FModelParameters.Scale});
      GLTreeSprite.Height:=  Round(MyTree.MaxY{FModelParameters.Scale});
      GLTreeSprite.Position.SetPoint(0,Round(MyTree.MaxY{FModelParameters.Scale}/2),0);
      GLMaterialLibrary.Materials[1].Material.Texture.Image.LoadFromFile(GlsDataPath+{'ztree1.tga'}MyModelParameters.TreeSpriteFile);
      GLTreeSprite.Material.MaterialLibrary:=SpriteGLMaterialLibrary;
      GLTreeSprite.Material.LibMaterialName:='Sprite';
      GLTreeSpriteLoaded:=True;
      {GLTreeSprite.Visible:=True;}
    end;
    {showmessage( Inttostr(Round(FTree.MinY)) +' , '+
    Inttostr(Round(FTree.MaxY)) +' , '+
    Inttostr(Round(FTree.TrunkLength))+' , '+
    Inttostr(Round(FModelParameters.Scale))  ); }

  { export the tree structure to a 3d format }
  if (not GlsUserCancelled) then
  begin
    { Call the initialization section  }
    if Initialize(MyTree.MaxX, MyTree.MaxY, MyTree.MaxZ, MyTree.TotalStems) = 0 then Exit
    else
    begin
      { Export the mesh structure to the FreeForm }
      {For i:= 0 to FModelParameters.BoleSplits -1 do
      if ExportTreeMesh(FTree.Trunk[i]) = 0 then}
      if ExportTreeMesh(MyTree.Trunk) = 0 then
      begin
        MessageDlg('Error computing tree mesh!', mtError, [mbOK], 0);
        Exit;
      end;
    end;
    { Free the memory occupied by the internal representation of the tree
      When the form is Closed  TreePlanted:=True;}
    {Freeform data changed}
    ProgressBar.Position := 0;
    TreeFF.StructureChanged;
    FruitFF.StructureChanged;
    LeafFF.StructureChanged;
      for iii:=0 to (BigLeafFF.MeshObjects.Count-1) do
      begin
        mesh:=BigLeafFF.MeshObjects.Items[iii];
        mesh.Visible:=False;
      end;
    BigLeafFF.StructureChanged;
    FruitFF.Visible:=True;
    LeafFF.Visible:=True;
    BigLeafFF.Visible:=True;
    TreeFF.Visible:=True;
    {ZoomCentered;}
    DistanceActive:=True;
    DummyTree.Translate(0, -(MyTree.MaxY/2),0);
    FormMouseWheel(Self, [], Round(120*MyTree.MaxY), Point(0, 0), h);

    {Apply the Light Changes}
    SunHeightTB.Position:=MyModelParameters.SunHeight;
    GLLightSource2.Position.y:=MyModelParameters.SunHeight;
    SunStrengthTB.Position:=MyModelParameters.SunStrength;
    GLLightSource2.SpotCutOff:=MyModelParameters.SunStrength;
    SunFromTB.Position:=MyModelParameters.SunFrom;
    GLLightSource2.Position.x:=MyModelParameters.SunFrom;

    StemsLabel.Caption:=Inttostr(ViewerTotalStems);
    VectorLabel.Caption:=Inttostr(ViewerTotalVertices);
    LeavesLabel.Caption:=Inttostr(ViewerTotalLeaves);
    LeavesVLabel.Caption:=Inttostr(ViewerTotalLeaveVertices);
    FruitLabel.Caption:=Inttostr(ViewerTotalFruit);
    FruitVLabel.Caption:=Inttostr(ViewerTotalFruitVertices);
    BigTreeRatCountLabel.Caption:=Inttostr(BigTreeRatCount);
    {DistanceTrackBar.Position:= Round(FModelParameters.Distance);}
    DistanceTrackBar.Position:=Round(GLCamera.DistanceToTarget);
    DistanceActive:=False;
    ProgressBar.Position := 0;

{Showmessage(inttostr(LeafFF.MeshObjects.Count)+' : '+
inttostr(FruitFF.MeshObjects.Count)+' : '+
inttostr(TreeFF.MeshObjects.Count)); }

  end;
  dtmGlsTrealViewerForm.Cursor := crDefault;
end;


{Called in Load Parameters to AutoCenter}
function TdtmGlsTrealViewerForm.Initialize(MaxX, MaxY, MaxZ: real; ATotalStems: integer): integer;
begin
  ProgressBar.Max := ATotalStems+ MyTree.TotalGrownLeaves+ MyTree.TotalFruit - 1;
  {Need to CLEAR all the Trunk and Leaf Objects...}

  { TotalVertices and TotalFaces hold the total amount of vertices and faces
    in the entire tree }
  ViewerTotalVertices := 0;
{  ViewerTotalFaces := 0;}
  Bolebusted:=0;
  { TotalStems is used for the internal counting of the total number of stems,
    ATotalStems is used to initialize the progress bar of the progress form }
  ViewerTotalStems := 0;
  ViewerTotalLeaveVertices:= 0;
  ViewerTotalLeaves:= 0;
  ViewerTotalFruit:= 0;
  ViewerTotalFruitVertices:= 0;
  DecimalSeparator := '.';
  ThousandSeparator := #0;
  Result := 1;
end;


{A Stem is a bunch of circles (Sections) formed by points (Vertices)
             stacked as rings to form a 'cylinder'
             that slowly reduces in diameter}
function TdtmGlsTrealViewerForm.AddBole(BoleSections: TList): integer;
{ Pre: Sections <> nil
  Ret: 0, AddStem failed
       1, AddStem succeeded, a mesh describing the stem segment made
          up by the sections defined in Sections has been written to 'tree.inc'. }
var
  i, j: integer;                           // Loop variables
  LocalVertices: integer;                // The amount of vertices and
                                    //this was for POV:
                                    // faces respectively in the segment mesh
  LocalVerticesPerSection: integer;             // The amount of vertices in a
                                             // section (is the same for all
                                             // sections)
  LocalTotalSections: integer;                  // Total amount of sections
  Mesh	:TMeshObject; // Structure containing the actual data
  FaceGroup: TFGVertexIndexList;
  Sign, Variation, Red, Blue, Green: integer;
begin
  If BoleMade then begin
  ViewerTotalStems := ViewerTotalStems + 1;
  { Count the amount of sections }
  LocalTotalSections := BoleSections.Count;
  { Count the amount of vertices in the stem }
  LocalVerticesPerSection := TList(BoleSections[0]).Count;
  LocalVertices := LocalVerticesPerSection * LocalTotalSections;
  { Update the total amount of vertices in the tree }
  ViewerTotalVertices := ViewerTotalVertices + LocalVertices;

  Mesh:=TMeshObject.CreateOwned(TreeFF.MeshObjects); // Create the mesh
  Mesh.Mode := momFaceGroups;
  Mesh.Vertices.Clear;
    { Write vertices making up the mesh }
    { For every section making up the stem write the vertices from that section
      to the vertex list of the mesh object }
  for i := 0 to LocalTotalSections - 1 do
  begin
    for j := 0 to LocalVerticesPerSection - 1 do
    begin
      { Repeatedly write a vertex from the current section }
      Mesh.Vertices.add(TPoint3d(TList(BoleSections[i])[j]).x,
          TPoint3d(TList(BoleSections[i])[j]).y,
          TPoint3d(TList(BoleSections[i])[j]).z);
      {Try to change the normals of the end... no difference
      If j=0 then
      Mesh.Normals.Add(TPoint3d(TList(BoleSections[i])[1]).x,
          TPoint3d(TList(BoleSections[i])[1]).y,
          TPoint3d(TList(BoleSections[i])[1]).z)
      else if j=LocalVerticesPerSection - 1 then
      Mesh.Normals.Add(TPoint3d(TList(BoleSections[i])[j-1]).x,
          TPoint3d(TList(BoleSections[i])[j-1]).y,
          TPoint3d(TList(BoleSections[i])[j-1]).z)
      else}
      Mesh.Normals.Add(TPoint3d(TList(BoleSections[i])[j]).x,
          TPoint3d(TList(BoleSections[i])[j]).y,
          TPoint3d(TList(BoleSections[i])[j]).z);
      If MyModelParameters.TreeBarkFile = '' then
      begin
        {Mesh.Colors.Add(0,0,1,0);}
        { Calculate the red, green and blue components from the leafcolor variable }
        Red := MyModelParameters.TreeColor and $FF;
        Green := (MyModelParameters.TreeColor and $FF00) shr 8;
        Blue := (MyModelParameters.TreeColor and $FF0000) shr 16;

        if MyModelParameters.TreeColorVariation then
        begin
          { Add a small variation to the leaf color }
          if Random(2) = 0 then Sign := 1 else Sign := -1;
          Variation := Random(40);
          Red := Red + Sign * Variation;
          Green := Green + Sign * Variation;
          Blue := Blue + Sign * Variation;

          if Red < 0 then Red := 0;
          if Green < 0 then Green := 0;
          if Blue < 0 then Blue := 0;
        end;
        Mesh.Colors.Add((Red / $FF),(Green / $FF),(Blue / $FF),0);
      end;
        //set the tex coords to cover the whole mesh
      with Mesh.TexCoords do
      {This tiles the texture.. Base is still befuddled}
      begin  {0..1}     {horizontal s, vertical t}  {8 base cuts 1..10-2 =8}
        {Add( (j/(VerticesPerSection-1)), (i/TotalSections-1)); }

       If Bolebusted > 8 then
        Add( (j/(LocalVerticesPerSection-1)), i)
        else Add( (j/(LocalVerticesPerSection-1)), ((i*i)/64) {(i/9)})
       { else If Bolebusted > 7 then
         Add( (j/(LocalVerticesPerSection-1)), (Bolebusted+1)-(i/(Bolebusted)))
        else Add( (j/(LocalVerticesPerSection-1)), (i/4))}
      end;
    end;{j}
      {Change the 2 Link Texture Coord Normal to same as others
      this does not work
       0 and LocalVerticesPerSection - 1}
      {Mesh.Normals.SetNormals(TPoint3d(TList(BoleSections[i])[0]).x,
          TPoint3d(TList(BoleSections[i])[0]).y,
          TPoint3d(TList(BoleSections[i])[0]).z):=
      Mesh.Normals(TPoint3d(TList(BoleSections[i])[1]).x,
          TPoint3d(TList(BoleSections[i])[1]).y,
          TPoint3d(TList(BoleSections[i])[1]).z);}



    inc(Bolebusted);
  end; {i}

        //add a new facegroup
  facegroup := TFGVertexIndexList.CreateOwned(Mesh.FaceGroups);
     //and set the material
  If MyModelParameters.TreeBarkFile <> '' then
     facegroup.MaterialName := 'TreeBark';
    {Write the face indices }
    { For every section write the segment mesh. The outer loop has a maximum of
      TotalSections - 2 because the faces are built using points from section i
      and section i + 1}
  for i := 0 to LocalTotalSections - 2 do
  begin
    for j := 0 to LocalVerticesPerSection - 1 do
    begin
        { Repeatedly write two face indices creating a cubic little plane between
          four vertices   :
          VerticesPerSection * i + j: lower right corner of the first face
          VerticesPerSection * (i + 1) + j: upper right corner of the first face
          VerticesPerSection * i + (j + 1) mod VerticesPerSection: lower left corner of the first face
          VerticesPerSection * i + (j + 1) mod VerticesPerSection: lower left corner of the second face
          VerticesPerSection * (i + 1) + j: upper right corner of the second face
          VerticesPerSection * (i + 1) + (j + 1) mod VerticesPerSection: upper left corner of the second face }
      with facegroup do
      begin
        add((LocalVerticesPerSection * i + j));
        add((LocalVerticesPerSection * (i + 1) + j));
        add((LocalVerticesPerSection * i + (j + 1) mod LocalVerticesPerSection));
        add((LocalVerticesPerSection * i + (j + 1) mod LocalVerticesPerSection));
        add((LocalVerticesPerSection * (i + 1) + j));
        add((LocalVerticesPerSection * (i + 1) + (j + 1) mod LocalVerticesPerSection));
      end;
    end;
    {Add in the Missing LINK  J is not valid  This turn it facing wrong.. but it is there}
      with facegroup do
      begin
          add((LocalVerticesPerSection * i ));// lower right corner of the first face
          add((LocalVerticesPerSection * (i + 1)));// upper right corner of the first face
          add((LocalVerticesPerSection * i + (LocalVerticesPerSection-1)));// lower left corner of the first face
          add((LocalVerticesPerSection * i + (LocalVerticesPerSection-1)));// lower left corner of the second face
          add((LocalVerticesPerSection * (i + 1) ));// upper right corner of the second face
          add((LocalVerticesPerSection * (i + 1) + (LocalVerticesPerSection-1)));// upper left corner of the second face
      end;
  end;
    {These facegroups also have a range of rendering modes:
      fgmmTriangles, fgmmTriangleStrip, fgmmFlatTriangles and fgmmTriangleFan.}
  FaceGroup.Mode := fgmmTriangles;
  ProgressBar.StepIt;
  end;
  Result := 1;
end;

{A Stem is a bunch of circles (Sections) formed by points (Vertices)
             stacked as rings to form a 'cylinder'
             that slowly reduces in diameter}
function TdtmGlsTrealViewerForm.AddStem(Sections: TList): integer;
{ Pre: Sections <> nil
  Ret: 0, AddStem failed
       1, AddStem succeeded, a mesh describing the stem segment made
          up by the sections defined in Sections has been written to 'tree.inc'. }
var
  i, j: integer;                           // Loop variables
  LocalVertices: integer;                // The amount of vertices and
                                    //this was for POV:
                                    // faces respectively in the segment mesh
  LocalVerticesPerSection: integer;             // The amount of vertices in a
                                             // section (is the same for all
                                             // sections)
  LocalTotalSections: integer;                  // Total amount of sections
  Mesh	:TMeshObject; // Structure containing the actual data
  FaceGroup: TFGVertexIndexList;
  Sign, Variation, Red, Blue, Green: integer;
begin
  ViewerTotalStems := ViewerTotalStems + 1;
  { Count the amount of sections }
  LocalTotalSections := Sections.Count;
  { Count the amount of vertices in the stem }
  LocalVerticesPerSection := TList(Sections[0]).Count;
  LocalVertices := LocalVerticesPerSection * LocalTotalSections;
  { Update the total amount of vertices in the tree }
  ViewerTotalVertices := ViewerTotalVertices + LocalVertices;

  Mesh:=TMeshObject.CreateOwned(TreeFF.MeshObjects); // Create the mesh
  Mesh.Mode := momFaceGroups;
  Mesh.Vertices.Clear;
    { Write vertices making up the mesh }
    { For every section making up the stem write the vertices from that section
      to the vertex list of the mesh object }
  for i := 0 to LocalTotalSections - 1 do
  begin
    for j := 0 to LocalVerticesPerSection - 1 do
    begin
      { Repeatedly write a vertex from the current section }
      Mesh.Vertices.add(TPoint3d(TList(Sections[i])[j]).x,
          TPoint3d(TList(Sections[i])[j]).y,
          TPoint3d(TList(Sections[i])[j]).z);
      {Try to change the normals of the ensd... no diference
      If j=0 then
      Mesh.Normals.Add(TPoint3d(TList(Sections[i])[1]).x,
          TPoint3d(TList(Sections[i])[1]).y,
          TPoint3d(TList(Sections[i])[1]).z)
      else if j=LocalVerticesPerSection - 1 then
      Mesh.Normals.Add(TPoint3d(TList(Sections[i])[j-1]).x,
          TPoint3d(TList(Sections[i])[j-1]).y,
          TPoint3d(TList(Sections[i])[j-1]).z)
      else}
      Mesh.Normals.Add(TPoint3d(TList(Sections[i])[j]).x,
          TPoint3d(TList(Sections[i])[j]).y,
          TPoint3d(TList(Sections[i])[j]).z);
      If MyModelParameters.TreeBarkFile = '' then
      begin
        {Mesh.Colors.Add(0,0,1,0);}
        { Calculate the red, green and blue components from the leafcolor variable }
        Red := MyModelParameters.TreeColor and $FF;
        Green := (MyModelParameters.TreeColor and $FF00) shr 8;
        Blue := (MyModelParameters.TreeColor and $FF0000) shr 16;

        if MyModelParameters.TreeColorVariation then
        begin
          { Add a small variation to the leaf color }
          if Random(2) = 0 then Sign := 1 else Sign := -1;
          Variation := Random(40);
          Red := Red + Sign * Variation;
          Green := Green + Sign * Variation;
          Blue := Blue + Sign * Variation;

          if Red < 0 then Red := 0;
          if Green < 0 then Green := 0;
          if Blue < 0 then Blue := 0;
        end;
        Mesh.Colors.Add((Red / $FF),(Green / $FF),(Blue / $FF),0);
      end;
        //set the tex coords to cover the whole mesh
      with Mesh.TexCoords do
      {This tiles the texture.. Base is still befuddled}
      begin  {0..1}     {horizontal s, vertical t}  {8 base cuts 1..10-2 =8}
        {Add( (j/(VerticesPerSection-1)), (i/TotalSections-1)); }

       If Bolebusted > 8 then
        Add( (j/(LocalVerticesPerSection-1)), i)
        else Add( (j/(LocalVerticesPerSection-1)), ((i*i)/64) {(i/9)})
       { else If Bolebusted > 7 then
         Add( (j/(LocalVerticesPerSection-1)), (Bolebusted+1)-(i/(Bolebusted)))
        else Add( (j/(LocalVerticesPerSection-1)), (i/4))}
      end;
    end;{j}
      {Change the 2 Link Texture Coord Normal to same as others
      this does not work
       0 and LocalVerticesPerSection - 1}
      {Mesh.Normals.SetNormals(TPoint3d(TList(Sections[i])[0]).x,
          TPoint3d(TList(Sections[i])[0]).y,
          TPoint3d(TList(Sections[i])[0]).z):=
      Mesh.Normals(TPoint3d(TList(Sections[i])[1]).x,
          TPoint3d(TList(Sections[i])[1]).y,
          TPoint3d(TList(Sections[i])[1]).z);}

    inc(Bolebusted);
  end; {i}

        //add a new facegroup
  facegroup := TFGVertexIndexList.CreateOwned(Mesh.FaceGroups);
     //and set the material
  If MyModelParameters.TreeBarkFile <> '' then
     facegroup.MaterialName := 'TreeBark';
    {Write the face indices }
    { For every section write the segment mesh. The outer loop has a maximum of
      TotalSections - 2 because the faces are built using points from section i
      and section i + 1}
  for i := 0 to LocalTotalSections - 2 do
  begin
    for j := 0 to LocalVerticesPerSection - 1 do
    begin
        { Repeatedly write two face indices creating a cubic little plane between
          four vertices   :
          VerticesPerSection * i + j: lower right corner of the first face
          VerticesPerSection * (i + 1) + j: upper right corner of the first face
          VerticesPerSection * i + (j + 1) mod VerticesPerSection: lower left corner of the first face
          VerticesPerSection * i + (j + 1) mod VerticesPerSection: lower left corner of the second face
          VerticesPerSection * (i + 1) + j: upper right corner of the second face
          VerticesPerSection * (i + 1) + (j + 1) mod VerticesPerSection: upper left corner of the second face }
      with facegroup do
      begin
        add((LocalVerticesPerSection * i + j));
        add((LocalVerticesPerSection * (i + 1) + j));
        add((LocalVerticesPerSection * i + (j + 1) mod LocalVerticesPerSection));
        add((LocalVerticesPerSection * i + (j + 1) mod LocalVerticesPerSection));
        add((LocalVerticesPerSection * (i + 1) + j));
        add((LocalVerticesPerSection * (i + 1) + (j + 1) mod LocalVerticesPerSection));
      end;
    end;
    {Add in the Missing LINK  J is not valid  This turn it facing wrong.. but it is there}
      with facegroup do
      begin
          add((LocalVerticesPerSection * i ));// lower right corner of the first face
          add((LocalVerticesPerSection * (i + 1)));// upper right corner of the first face
          add((LocalVerticesPerSection * i + (LocalVerticesPerSection-1)));// lower left corner of the first face
          add((LocalVerticesPerSection * i + (LocalVerticesPerSection-1)));// lower left corner of the second face
          add((LocalVerticesPerSection * (i + 1) ));// upper right corner of the second face
          add((LocalVerticesPerSection * (i + 1) + (LocalVerticesPerSection-1)));// upper left corner of the second face
      end;
  end;
    {These facegroups also have a range of rendering modes:
      fgmmTriangles, fgmmTriangleStrip, fgmmFlatTriangles and fgmmTriangleFan.}
  FaceGroup.Mode := fgmmTriangles;
  ProgressBar.StepIt;
  Result := 1;
end;


{Each Leaf is a closed Polygon...
 use Mesh to place ALL leaves into the Freeform
 The Real LOD advantage would be to make 1 Leaf and Proxies the rest...}
function TdtmGlsTrealViewerForm.AddLeaf(Points: TList; LeafColor: integer; LeafColorVariation: boolean): integer;
var  i: integer;
    Sign: integer;
    Red, Blue, Green: integer;
    Variation: integer;
  Mesh	:TMeshObject; // Structure containing the actual data
  FaceGroup: TFGVertexIndexList;
begin
  ViewerTotalLeaves := ViewerTotalLeaves + 1;
  If DisplayWithLeaves then
  begin
    {Switch depending if LSF or 3ds file}
     inc(LeafShapeRotator);
     If LeafShapeRotator >= MyModelParameters.LeafShapeCount  then
        LeafShapeRotator:=0;
                               {TreeFreeForm TreeLeafFF}
    Mesh:=TMeshObject.CreateOwned(LeafFF.MeshObjects); // Create the mesh
    Mesh.Mode := momFaceGroups; {(momTriangles, momTriangleStrip, momFaceGroups);}
    Mesh.Vertices.Clear;
    ViewerTotalLeaveVertices:=ViewerTotalLeaveVertices+Points.Count;
    for i := 0 to Points.Count - 1 do
    begin
      Mesh.Vertices.add (TPoint3d(Points[i]).x,
                         TPoint3d(Points[i]).y,
                         TPoint3d(Points[i]).z);
      Mesh.Normals.Add(TPoint3d(Points[i]).x,
                       TPoint3d(Points[i]).y,
                       TPoint3d(Points[i]).z);
      If MyModelParameters.LeafTextureCount=0then
      {If FModelParameters.LeafTextureFile[0] = '' then}
      begin
        { Calculate the red, green and blue components from the leafcolor variable }
        {This is broke.. also need to make Front and Bacl Leaf colors}
        Red := LeafColor and $FF;
        Green := (LeafColor and $FF00) shr 8;
        Blue := (LeafColor and $FF0000) shr 16;
        if LeafColorVariation then
        begin
          { Add a small variation to the leaf color }
          if Random(2) = 0 then Sign := 1 else Sign := -1;
          Variation := Random(40);
          Red := Red + Sign * Variation;
          Green := Green + Sign * Variation;
          Blue := Blue + Sign * Variation;
          if Red < 0 then Red := 0;
          if Green < 0 then Green := 0;
          if Blue < 0 then Blue := 0;
        end;
        Mesh.Colors.Add((Red / $FF),(Green / $FF),(Blue / $FF),0);
      end;
      {LeafTexture}
       //set the tex coords to cover the whole mesh
      with Mesh.TexCoords do   {This does Not tile the texture,corrects offset}
      begin       {horizonatl s, vertical t}
        Add( LeafPolygonVertices[LeafShapeRotator,i,0]+0.5,
              LeafPolygonVertices[LeafShapeRotator,i,1]);
      end;
    end;
    //add a new facegroup
    facegroup := TFGVertexIndexList.CreateOwned(Mesh.FaceGroups);
      //and set the material
    If MyModelParameters.LeafTextureCount>0then
    {If MyModelParameters.LeafTextureFile[0] <> '' then}
        facegroup.MaterialName := 'LeafTexture'+Inttostr(LeafShapeRotator);

        {Leaves are clockwise Triangle fans: make triangles}
    for i := 1 to Points.Count - 3 do
    with facegroup do
    begin
      add(0); add(i ); add(i+1);
      add(0); add(i + 1); add(i+2);
    end;
      {These facegroups also have a range of rendering modes:
      fgmmTriangles, fgmmTriangleStrip, fgmmFlatTriangles and fgmmTriangleFan.}
    FaceGroup.Mode := fgmmTriangleFan;   {Winding clockwise (reversed) glFrontFace}
  end;
  ProgressBar.StepIt;
  Result := 1;
end;

function TdtmGlsTrealViewerForm.AddBigLeaf(Points: TList; LeafColor: integer; LeafColorVariation: boolean): integer;
var  i: integer;
{    Sign: integer;}
    Red, Blue, Green: integer;
{    Variation: integer;}
  Mesh	:TMeshObject; // Structure containing the actual data
  FaceGroup: TFGVertexIndexList;
begin
  {Create BIG leaves for Range degradation}
  {Make them all, Percentage done in GlsTree BigLeaf creation}
  {Switch depending if LSF or 3ds file}
  begin
    inc(BigTreeRatCount);
    Mesh:=TMeshObject.CreateOwned(BigLeafFF.MeshObjects); // Create the mesh
    Mesh.Mode := momFaceGroups; {(momTriangles, momTriangleStrip, momFaceGroups);}
    Mesh.Vertices.Clear;
    for i := 0 to Points.Count - 1 do
    begin
      Mesh.Vertices.add (TPoint3d(Points[i]).x,
                         TPoint3d(Points[i]).y,
                         TPoint3d(Points[i]).z);
      Mesh.Normals.Add(TPoint3d(Points[i]).x,
                       TPoint3d(Points[i]).y,
                       TPoint3d(Points[i]).z);
      {If FModelParameters.LeafTextureFile[0] = '' then}
      If MyModelParameters.LeafTextureCount=0then
      begin
        Red := LeafColor and $FF;
        Green := (LeafColor and $FF00) shr 8;
        Blue := (LeafColor and $FF0000) shr 16;
        Mesh.Colors.Add((Red / $FF),(Green / $FF),(Blue / $FF),0);
      end;
      with Mesh.TexCoords do   {This does Not tile the texture,corrects offset}
      begin       {horizonatl s, vertical t}
        Add( LeafPolygonVertices[0,i,0]+0.5,
              LeafPolygonVertices[0,i,1]);
      end;
    end;
    //add a new facegroup
    facegroup := TFGVertexIndexList.CreateOwned(Mesh.FaceGroups);
      //and set the material
    {If FModelParameters.LeafTextureFile[0] <> '' then}
    If MyModelParameters.LeafTextureCount>0then
        facegroup.MaterialName := 'LeafTexture0';
        {Leaves are clockwise Triangle fans: make triangles}
    for i := 1 to Points.Count - 3 do
    with facegroup do
    begin
      add(0); add(i ); add(i+1);
      add(0); add(i + 1); add(i+2);
    end;
      {These facegroups also have a range of rendering modes:
      fgmmTriangles, fgmmTriangleStrip, fgmmFlatTriangles and fgmmTriangleFan.}
    FaceGroup.Mode := fgmmTriangleFan;   {Winding clockwise (reversed) glFrontFace}
  end;
  Result := 1;
end;

{Each Fruit is a closed Polygon...
 use Mesh to place ALL Fruit into the Freeform
 The Real LOD advantage would be to make 1 Fruit and Proxies the rest...}
function TdtmGlsTrealViewerForm.AddFruit(Points: TList; FruitColor: integer; FruitColorVariation: boolean): integer;
var  i: integer;
    Sign: integer;
    Red, Blue, Green: integer;
    Variation: integer;
  Mesh	:TMeshObject; // Structure containing the actual data
  FaceGroup: TFGVertexIndexList;
begin
  ViewerTotalFruit := ViewerTotalFruit + 1;
  If DisplayWithLeaves then
  begin
    {Switch depending if LSF or 3ds file}
                               {LeafFF FruitFF}
     inc(FruitShapeRotator);
     If FruitShapeRotator >= MyModelParameters.FruitShapeCount  then
        FruitShapeRotator:=0;
    Mesh:=TMeshObject.CreateOwned(FruitFF.MeshObjects); // Create the mesh
    Mesh.Mode := momFaceGroups; {(momTriangles, momTriangleStrip, momFaceGroups);}
    Mesh.Vertices.Clear;
    ViewerTotalFruitVertices:=ViewerTotalFruitVertices+Points.Count;
    for i := 0 to Points.Count - 1 do
    begin
      Mesh.Vertices.add (TPoint3d(Points[i]).x,
                         TPoint3d(Points[i]).y,
                         TPoint3d(Points[i]).z);
      Mesh.Normals.Add(TPoint3d(Points[i]).x,
                       TPoint3d(Points[i]).y,
                       TPoint3d(Points[i]).z);

      If MyModelParameters.FruitTextureCount = 0 then
      begin
        { Calculate the red, green and blue components from the leafcolor variable }
        {This is broke.. also need to make Front and Back Leaf colors}
        Red := FruitColor and $FF;
        Green := (FruitColor and $FF00) shr 8;
        Blue := (FruitColor and $FF0000) shr 16;
        if FruitColorVariation then
        begin
          { Add a small variation to the leaf color }
          if Random(2) = 0 then Sign := 1 else Sign := -1;
          Variation := Random(40);
          Red := Red + Sign * Variation;
          Green := Green + Sign * Variation;
          Blue := Blue + Sign * Variation;
          if Red < 0 then Red := 0;
          if Green < 0 then Green := 0;
          if Blue < 0 then Blue := 0;
        end;
        Mesh.Colors.Add((Red / $FF),(Green / $FF),(Blue / $FF),0);
      end;
      {LeafTexture}
       //set the tex coords to cover the whole mesh
      with Mesh.TexCoords do   {This does Not tile the texture,corrects offset}
      begin       {horizonatl s, vertical t}
        Add( FruitPolygonVertices[FruitShapeRotator,i,0]+0.5,
              FruitPolygonVertices[FruitShapeRotator,i,1]);
      end;
    end;
    //add a new facegroup
    facegroup := TFGVertexIndexList.CreateOwned(Mesh.FaceGroups);
      //and set the material
    If MyModelParameters.FruitTextureCount > 0 then
        facegroup.MaterialName := 'FruitTexture'+Inttostr(FruitShapeRotator);
        {Fruit are clockwise Triangle fans: make triangles}
    for i := 1 to Points.Count - 3 do
    with facegroup do
    begin
      add(0); add(i ); add(i+1);
      add(0); add(i + 1); add(i+2);
    end;
      {These facegroups also have a range of rendering modes:
      fgmmTriangles, fgmmTriangleStrip, fgmmFlatTriangles and fgmmTriangleFan.}
    FaceGroup.Mode := fgmmTriangleFan;   {Winding clockwise (reversed) glFrontFace}
  end;
  ProgressBar.StepIt;
  Result := 1;
end;


procedure TdtmGlsTrealViewerForm.SaveasGLSM1Click(Sender: TObject);
//Create a NEW Freeform to HOLD the Tree... then delete it
begin
  SaveDialog.InitialDir:= DXFPath;
  SaveDialog.Filter:='*.glsm';  //GLS file format
  //SaveDialog.FileName:=TreeShapeEdit.Text;
  if SaveDialog.Execute then
  begin
    Application.ProcessMessages;
    //DistanceTrackBarChange
    //LoadParameters
    //COMBINE into 1 : TreeFF   FruitFF   LeafFF
    TreeFF.SaveToFile(SaveDialog.FileName);
    //TreeFF.BuildMesh(GLFreeForm1);
    GLMaterialLibrary.SaveToFile(ChangeFileExt(SaveDialog.FileName,'.glml'));
  end;
end;

end.
